literary_styles = [
    'Fiction',
    'Non-fiction',
    'Poetry',
    'Drama',
    'Prose',
    'Satire',
    'Comedy',
    'Tragedy',
    'Epic',
    'Sonnet',
    'Haiku',
    'Lyric',
    'Narrative',
    'Mystery',
    'Horror',
    'Romance',
    'Historical fiction',
    'Science fiction',
    'Fantasy',
    'Realism',
    'Naturalism',
    'Modernism',
    'Postmodernism',
    'Magical realism',
    'Surrealism',
    'Stream of consciousness',
    'Gothic',
    'Dystopian',
    'Bildungsroman',
    'Picaresque',
    'Memoir',
    'Autobiography',
    'Biography',
    'Travel writing',
    'Essay',
    'Journal',
    'Criticism',
    'Manifesto',
    'Satire',
    'Parody',
    'Fable',
    'Allegory',
    'Myth',
    'Legend',
    'Folklore',
    'Fairy tale',
    'Nursery rhyme',
    'Ballad',
    'Limerick',
    'Epistolary',
]
surrealistic_adjectives = [
    'Jumbo',
    'Whispering',
    'Glowing',
    'Inflatable',
    'Cactus',
    'Crispy',
    'Billowing',
    'Bouncing',
    'Rotating',
    'Swirling',
    'Sticky',
    'Blazing',
    'Fizzy',
    'Jagged',
    'Glimmering',
    'Wobbling',
    'Tangled',
    'Hissing',
    'Luminous',
    'Crunchy',
    'Gaseous',
    'Serpentine',
    'Bumpy',
    'Flimsy',
    'Spinning',
    'Sizzling',
    'Blinking',
    'Fluffy',
    'Crinkly',
    'Rustling',
    'Sparkling',
    'Quivering',
    'Zesty',
    'Dangling',
    'Glazed',
    'Drifting',
    'Illuminated',
    'Dizzy',
    'Wavy',
    'Chirping',
    'Smoldering',
    'Quicksilver',
    'Splintered',
    'Gurgling',
    'Wispy',
    'Stuttering',
    'Thunderous',
    'Gossamer',
    'Spiky',
    'Oscillating'
]
surrealistic_nouns = [
    'Lighthouse',
    'Trampoline',
    'Kaleidoscope',
    'Windmill',
    'Dandelion',
    'Waterfall',
    'Wristwatch',
    'Jellyfish',
    'Chandelier',
    'Zucchini',
    'Tambourine',
    'Thunderbolt',
    'Seashell',
    'Saxophone',
    'Hammock',
    'Mandolin',
    'Traffic light',
    'Helicopter',
    'Feather boa',
    'Teapot',
    'Hula hoop',
    'Pinwheel',
    'Folding fan',
    'Ocarina',
    'Slinky',
    'Salt shaker',
    'Harmonica',
    'Teardrop',
    'Flashlight',
    'Disco ball',
    'Electric fan',
    'Barometer',
    'Microscope',
    'Fire hydrant',
    'Kite',
    'Hairbrush',
    'Megaphone',
    'Umbrella',
    'Paper clip',
    'Stethoscope',
    'Vase',
    'Tumbleweed',
    'Hourglass',
    'Cuckoo clock',
    'Sundial',
    'Trombone',
    'Accordion',
    'Tennis racket',
    'Molotov cocktail',
    'Banjo',
    'Candelabra'
]
